﻿//履歴を表示して思い出に浸るプラグインですｗ
//履歴が沢山ないと思い出に浸れません(´・ω・｀)
//「■設定」の部分の値を変えて再コンパイルすればイロイロ調整できます・・
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using System.Text;
using System.IO;
using LiveViewer;

namespace Plugin_GOA_Memory {
    public class Plugin_GOA_Memory : ISitePlugin {
        #region ■設定

        const bool GOAONLY     = true;  //true:GOAのみ、false:全サイト
        const bool RNDLOCATION = true;  //true:小窓の表示位置をランダムに、false：既定位置
        const int  MAXPEFS     =   20;  //仮想オンライン人数(最大)
        const int  MINPEFS     =    5;  //仮想オンライン人数(最小)
        const int  OUTRATE     =   50;  //オンラインをOUTにする確率(1～100)
        const int  WAITDATE    = 4000;  //メッセージの表示時間:最初の時刻表示(単位:ms)
        const int  WAITBASE    = 1000;  //メッセージの表示時間:基本(0文字の場合)(単位:ms)
        const int  WAITADD     =  100;  //メッセージの表示時間:加算(1文字増える毎に加算)(単位:ms)
        const long DIVIDESPAN  =  120;  //メッセージの間隔がこれ以上開いたら区切る(始点・終点)(単位:min)

        #endregion


        #region ■定義

        private class FormFlashGOAMemory : FormFlash {
            static Random Rnd   = new Random();

            Timer         Timer = new Timer();
            Bitmap        Sumbnail;

            int           MessagePos;       //表示中のメッセージ番号
            int           MessageEnd;       //終了する位置
            int           MessageCount = 0; //メッセージを表示した回数

            RectangleF    MesRectangleA;
            RectangleF    MesRectangleB;
            StringFormat  MesStringFormat;
            Font          MesFont;

            public FormFlashGOAMemory(Performer pef)
                : base(pef) {
                //イロイロ変更・・
                Opacity              = 0.0;
                Flash.Visible        = false;
                PanelViewOut.Visible = false;
                DoubleBuffered       = true;
                MaximizeBox          = false;
                BackColor            = Color.Lavender;
                FormBorderStyle      = FormBorderStyle == FormBorderStyle.None ? FormBorderStyle.None : FormBorderStyle.FixedSingle;
                Shown               += new EventHandler(FormFlashGOAMemory_Shown);
                FormClosed          += new FormClosedEventHandler(FormFlashGOAMemory_FormClosed);

                //小窓の位置変更
                if (RNDLOCATION) {
                    this.StartPosition = FormStartPosition.Manual;
                    Location = new Point(Rnd.Next(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width - Width), Rnd.Next(System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height - Height));
                }

                PerformerLog pl = pef.Tag as PerformerLog;
                if (pl == null) {
                    //履歴やお気に入りから開いた場合・・情報不足なので表示できません^^;
                    MessageBox.Show("オンライン一覧からしか開けません＞＜");
                } else {
                    //画像の準備
                    try {
                        using (Image img = Image.FromFile(pef.ImagePath)) {
                            Sumbnail = new Bitmap(ClientSize.Width, ClientSize.Height);
                            using (Graphics g = Graphics.FromImage(Sumbnail)) {
                                g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                                g.DrawImage(img, 0, 0, ClientSize.Width, ClientSize.Height);
                            }
                        }
                    } catch {
                        Sumbnail = null;
                    }

                    Timer.Tick    += new EventHandler(FlashFormTimer_Tick);
                    Timer.Tag      = this;
                    Timer.Interval = WAITDATE;

                    //ログの表示開始位置を決定する(区切りの良さそうな位置を探してその中からランダムに選ぶ)
                    List<int> startPoints = new List<int>();
                    startPoints.Add(0);
                    for (int i = 1; i < pl.Logs.Count; i++) {
                        if (pl.Logs[i].T - pl.Logs[i - 1].T > (DIVIDESPAN * 60 * 1000 * 1000 * 10)) {
                            startPoints.Add(i);
                        }
                    }
                    int point = Rnd.Next(startPoints.Count);
                    MessagePos = startPoints[point];
                    if (startPoints.Count == point + 1) {
                        MessageEnd = pl.Logs.Count;
                    } else {
                        MessageEnd = startPoints[point + 1];
                    }

                    //日時を表示
                    DateTime dt  = new DateTime(pl.Logs[MessagePos].T);
                    string   mes = dt.ToString("yyyy年\r\nMM月dd日\r\nHH時mm分\r\nddd曜");

                    using (Font font = new Font("ＭＳ ゴシック", 42.0f / 320.0f * (float)Width, FontStyle.Regular))
                    using (StringFormat sf = new StringFormat()) {
                        sf.Alignment     = StringAlignment.Center;
                        sf.LineAlignment = StringAlignment.Center;

                        Bitmap bmp = new Bitmap(ClientSize.Width, ClientSize.Height);
                        using (Graphics g = Graphics.FromImage(bmp)) {
                            g.DrawString(mes, font, Brushes.CornflowerBlue, new RectangleF(4, 4, ClientSize.Width - 8, ClientSize.Height - 8), sf);
                            g.DrawString(mes, font, Brushes.DarkBlue,       new RectangleF(0, 0, ClientSize.Width - 8, ClientSize.Height - 8), sf);
                        }
                        BackgroundImage = bmp;
                    }
                }
            }

            private void FormFlashGOAMemory_Shown(object sender, EventArgs e) {
                //メッセージのフォントなど
                MesRectangleA   = new RectangleF(5, 5, ClientSize.Width - 8, ClientSize.Height - 8);
                MesRectangleB   = new RectangleF(4, 4, ClientSize.Width - 8, ClientSize.Height - 8);
                MesFont         = new Font("ＭＳ ゴシック", 12.0f / 320.0f * (float)Width, FontStyle.Bold);
                MesStringFormat = new StringFormat();
                MesStringFormat.Alignment = StringAlignment.Center;
                Timer.Enabled = true;

                //フェードインｗ
                for (double d = 0.099999999999999; d < 1.0; d += 0.01) {
                    Opacity = d;
                    Application.DoEvents();
                    System.Threading.Thread.Sleep(10);
                }
            }

            private void FormFlashGOAMemory_FormClosed(object sender, FormClosedEventArgs e) {
                MesFont.Dispose();
                MesStringFormat.Dispose();
                Timer.Dispose();
            }

            private void FlashFormTimer_Tick(object sender, EventArgs e) {
                Timer              t  = sender           as Timer;
                FormFlashGOAMemory ff = t.Tag            as FormFlashGOAMemory;
                PerformerLog       pl = ff.Performer.Tag as PerformerLog;

                if (ff.MessagePos >= pl.Logs.Count || ff.MessagePos == ff.MessageEnd){
                   //(ff.MessageCount > 0 && ff.MessagePos > 0 && (pl.Logs[ff.MessagePos].T - pl.Logs[ff.MessagePos - 1].T) > (CLOSESPAN * 1000 * 1000 * 60))) {
                    //最後のメッセージまできた
                    if (ff.MessageCount == 0) {
                        //まだ１度もメッセージ表示してないので最初のメッセージから＾＾；
                        ff.MessagePos = 0;
                    } else {
                        //フェードアウトｗ
                        for (double d = ff.Opacity; d >= 0.0; d -= 0.01) {
                            ff.Opacity = d;
                            Application.DoEvents();
                            System.Threading.Thread.Sleep(10);
                        }

                        //窓を閉じる
                        ff.Close();
                    }
                } else {
                    string sMessage = pl.Logs[ff.MessagePos].M;
                    if (sMessage.Length > 0 && sMessage[0] == '≫') {
                        //メッセージを表示する
                        ff.MessageCount++;
                        t.Interval = WAITBASE + (sMessage.Length * WAITADD);

                        Bitmap bmp;
                        if (Sumbnail == null) bmp = new Bitmap(ff.ClientSize.Width, ff.ClientSize.Height);
                        else                  bmp = new Bitmap(Sumbnail);

                        using (Graphics g = Graphics.FromImage(bmp)) {
                            g.DrawString(sMessage.Substring(1), MesFont, Brushes.Black, MesRectangleA, MesStringFormat);
                            g.DrawString(sMessage.Substring(1), MesFont, Brushes.Red,   MesRectangleB, MesStringFormat);
                        }

                        Image tmp = ff.BackgroundImage;
                        ff.BackgroundImage = bmp;
                        tmp.Dispose();
                    } else {
                        //メッセージ以外なので読み飛ばす
                        t.Interval = 100;
                    }
                    ff.MessagePos++;
                }
            }
        }
        
        #endregion


        #region ■オブジェクト

        private Random             Rnd        = new Random();             //乱数
        private List<PerformerLog> TargetLogs = new List<PerformerLog>(); //表示対象のパフォログ一覧
        private List<Performer>    OnlinePefs = new List<Performer>();    //仮想オンラインパフォ一覧
        
        #endregion


        #region ■ISitePluginインターフェースの実装

        public string Site       { get { return "GOAの記憶"; } }

        public string Caption    { get { return "GOAの想い出に浸るプラグイン(2008/09/30版)"; } }

        public string TopPageUrl { get { return "http://web.archive.org/web/*/http://www.girlsonair.tv"; } }

        public void Begin() {
            //履歴から表示対象だけ抜き出す（GOAでメッセージ履歴がある）
            foreach (PerformerLog pl in Pub.ViewLog.Values) {
                if (pl.Logs.Count > 0) {
                    if (!GOAONLY || pl.Site == "GOA") {
                        TargetLogs.Add(pl);
                    }
                }
            }

            //GOAの履歴がなければ全サイトを対象に
            if (TargetLogs.Count == 0) {
                foreach (PerformerLog pl in Pub.ViewLog.Values) {
                    if (pl.Logs.Count > 0) {
                        TargetLogs.Add(pl);
                    }
                }

                //それでもなければエラー表示
                if (TargetLogs.Count == 0) {
                    MessageBox.Show("メッセージ履歴が存在しません＞＜");
                }
            }
        }

        public void End() {
        }

        public List<Performer> Update() {
            //一定割合でOUTさせる(開いている窓は対象外)
            List<Performer> outs = new List<Performer>();
            foreach (Performer p in OnlinePefs) {
                if (!Pub.FlashForms.ContainsKey(p.Key)) {
                    if (Rnd.NextDouble() <= (double)OUTRATE / 100.0) {
                        outs.Add(p);
                    }
                }
            }
            foreach (Performer p in outs) {
                OnlinePefs.Remove(p);
            }

            //ランダムな人数に調整
            int iOnline = Rnd.Next(MINPEFS, MAXPEFS);
            if (iOnline > OnlinePefs.Count) {
                //目標値が現状より多い→INさせる
                int n = iOnline - OnlinePefs.Count;
                for (int i = 0; i < n; i++) {
                    PerformerLog pl = TargetLogs[Rnd.Next(TargetLogs.Count)];
                    if (OnlinePefs.Find(delegate(Performer pef) { return pef.ID == pl.ID; }) == null) {
                        Performer p = new Performer(this, pl.ID);
                        p.Name      = pl.Name;
                        p.Age       = pl.Age;
                        p.Area      = pl.Area;
                        p.OtherInfo = pl.OtherInfo;
                        p.RoomName  = pl.Site;
                        p.ImageUrl  = pl.ImageUrl;
                        p.Tag       = pl;
                        OnlinePefs.Add(p);
                    }
                }
            } else {
                //目標値が現状より少ない→無視^^;
            }

            return OnlinePefs;
        }

        public FormFlash OpenFlash(Performer performer) {
            return new FormFlashGOAMemory(performer);
        }

        public string GetFlashUrl(Performer performer) {
            return "DUMMY"; //Flash不要なのでダミー＾＾；
        }

        public Clipping GetFlashClipping(Performer performer) {
            return null;
        }

        public string GetProfileUrl(Performer performer) {
            return "http://web.archive.org/web/*/http://www.girlsonair.tv/live/index.jsp?id=" + performer.ID;
        }

        #endregion
    }
}
