﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Net;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using System.CodeDom.Compiler;
using Microsoft.JScript;
using LiveViewer;
using LiveViewer.Tool;
using LiveViewer.HtmlParser;

namespace Plugin_LiveDe55 {
    public class Plugin_LiveDe55 : ISitePluginBase {
        #region ■定義

        private class FormFlashLiveDe55 : FormFlashBrowserBase {
            public FormFlashLiveDe55(Performer pef)
                : base(pef) {
                //右クリックメニュー登録
                ToolStripMenuItem OpenMovie = new ToolStripMenuItem("ムービー画面を開く");
                OpenMovie.Click += OpenMovie_Click;
                contextMenuStrip.Items.Add(new ToolStripSeparator());
                contextMenuStrip.Items.Add(OpenMovie);
            }

            protected override FormFlashBrowserBase.ReloadKind ReloadMode {
                get { return ReloadKind.BrowserGoBack; }
            }

            protected override FormFlashBrowserBase.SecurityKind SecurityMode {
                get { return SecurityKind.Maximum; }
            }

            protected override string BrowserUrl {
                get { return Performer.Plugin.TopPageUrl + "wait.php?id=" + Performer.ID; }
            }

            private void OpenMovie_Click(object sender, EventArgs e) {
                Pub.OpenBrowser(Performer.Plugin.TopPageUrl + "movie.php?id=" + Performer.ID);
            }
        }

        private class WebClientEx : WebClient {
            protected override WebRequest GetWebRequest(Uri address) {
                WebRequest wr = base.GetWebRequest(address);
                HttpWebRequest hwr = wr as HttpWebRequest;
                if (hwr != null) {
                    hwr.AutomaticDecompression = DecompressionMethods.GZip | DecompressionMethods.Deflate; //圧縮を有効化
                }
                return wr;
            }
        }

        #endregion


        #region ■オブジェクト

        private WebClient Client = new WebClientEx(); //HTML取得用

        private Type   JsExecuterType   = null;
        private object JsExecuterObject = null;
        private string JsSource         = @"
            package JSExecuter
            {
                public class JSExecuter {
                    public function Eval(sJsCode : String) : Object { 
                        return eval(sJsCode);
                    }
                }
            }
        ";

        #endregion


        #region ■ISitePluginBaseインターフェースの実装

        public string Site       { get { return "livede55"; } }

        public string Caption    { get { return "ライブでゴーゴー用のプラグイン(2011/11/30版)"; } }

        public string TopPageUrl { get { return "http://livede55.com/"; } }

        public void Begin() {
            //プラグイン開始時処理

            //コンパイルするための準備
            JScriptCodeProvider jcp = new JScriptCodeProvider();

            //コンパイルパラメータ（メモリ内で生成）
            string[]           assemblys = new string[] { Assembly.GetAssembly(this.GetType()).Location };
            CompilerParameters cp        = new CompilerParameters(assemblys);
            cp.GenerateInMemory = true;

            //コンパイル
            CompilerResults cres = jcp.CompileAssemblyFromSource(cp, JsSource);

            //コンパイルしたアセンブリを取得
            Assembly asm = cres.CompiledAssembly;

            //クラスのTypeを取得
            JsExecuterType = asm.GetType("JSExecuter.JSExecuter");

            //インスタンスの作成
            JsExecuterObject = Activator.CreateInstance(JsExecuterType);

            //User-Agentを設定
            Client.Headers.Add(HttpRequestHeader.UserAgent, Pub.UserAgent + "_" + Site);
        }

        public void End() {
            //プラグイン終了時処理
        }

        public List<Performer> Update() {
            List<Performer> pefs = new List<Performer>();
            bool            ok   = false;

            //Party・2Shotルームの処理
            try {
                //WebからJsファイルを読み取る
                StringBuilder sb = new StringBuilder();
                using (Stream s = Client.OpenRead(TopPageUrl + "api/onair_json2.php")) {
                    using (StreamReader sr = new StreamReader(s)) {
                        sb.Append(sr.ReadToEnd());
                    }
                }
                string sJson = sb.ToString() + ";";

                //Jsファイルの内容を実行する
                ArrayObject obj = JsExecuterType.InvokeMember("Eval", BindingFlags.InvokeMethod, null, JsExecuterObject, new object[] { sJson }) as ArrayObject;
                for (int i = 0; i < (int)obj.length; i++) {
                    JSObject jso = obj[i] as JSObject;
                    string sHash      = (string)jso.GetField("hash",      BindingFlags.Default).GetValue(null);
                    string sNick_Name = (string)jso.GetField("nick_name", BindingFlags.Default).GetValue(null);
                    string sImg       = (string)jso.GetField("img",       BindingFlags.Default).GetValue(null);
                    int    iMic       = (int)   jso.GetField("mic",       BindingFlags.Default).GetValue(null);
                    int    iNewFace   = (int)   jso.GetField("newface",   BindingFlags.Default).GetValue(null);
/*★*/              int    iArea      = (int)   jso.GetField("area",      BindingFlags.Default).GetValue(null);
                    int    iQuality   = (int)   jso.GetField("quality",   BindingFlags.Default).GetValue(null);
                    int    iVideo     = (int)   jso.GetField("video",     BindingFlags.Default).GetValue(null);
                    int    iEvent     = (int)   jso.GetField("event",     BindingFlags.Default).GetValue(null);
                    int    iParty     = (int)   jso.GetField("party",     BindingFlags.Default).GetValue(null);
                    int    iChat      = (int)   jso.GetField("chat",      BindingFlags.Default).GetValue(null);
                    int    iOnline    = (int)   jso.GetField("online",    BindingFlags.Default).GetValue(null);

                    Performer p = new Performer(this, sHash);
                    p.Name             = HttpUtilityEx.HtmlDecode(sNick_Name);
                    p.ImageUrl         = "http://photo.livede55.com/photo_index/" + sImg;
                    p.ImageUpdateCheck = false;

                    p.Dona     = (iChat    != 0);
                    p.TwoShot  = (iParty   == 0);
                    p.Mic      = (iMic     != 0);
                    if (iNewFace != 0) {
                        if (iNewFace == 1) p.Debut   = true;
                        else               p.NewFace = true;
                    }

                    if (p.TwoShot) {
                        p.RoomName = "2ｼｮｯﾄ";
                    } else {
                        p.RoomName = "ﾊﾟｰﾃｨｰ";
                    }

                    if (iEvent != 0) {
                        p.RoomName = "E-" + p.RoomName;
                    }

                    if (iQuality != 0) {
                        p.OtherInfo += "高画質 ";
                    }

                    if (iVideo != 0) {
                        p.OtherInfo += "紹介動画 ";
                    }

                    pefs.Add(p);
                }
                ok = true;
            } catch (Exception ex) {
                Log.Add(Site + "-Update失敗(onair)", ex.ToString(), LogColor.Error);
            }

            if (ok) {
                //DSルームの処理
                try {
                    //WebからJsファイルを読み取る
                    StringBuilder sb = new StringBuilder();
                    using (Stream s  = Client.OpenRead(TopPageUrl + "api/dsroom_json.php")) {
                        using (StreamReader sr = new StreamReader(s)) {
                            sb.Append(sr.ReadToEnd());
                        }
                    }
                    string sJson = "[" + sb.ToString() + "];";

                    //Jsファイルの内容を実行する
                    ArrayObject obj = JsExecuterType.InvokeMember("Eval", BindingFlags.InvokeMethod, null, JsExecuterObject, new object[] { sJson }) as ArrayObject;
                    JSObject    jso = obj[0] as JSObject;
                    
                    foreach (FieldInfo fi in jso.GetFields(BindingFlags.Default)) {
                        ArrayObject pair = fi.GetValue(jso) as ArrayObject;

                        JSObject jso1 = pair[0] as JSObject;
                        string sHash1      = (string)jso1.GetField("hash",      BindingFlags.Default).GetValue(null);
                        string sNick_Name1 = (string)jso1.GetField("nick_name", BindingFlags.Default).GetValue(null);
                        string sImg1       = (string)jso1.GetField("img",       BindingFlags.Default).GetValue(null);

                        Performer p1 = new Performer(this, sHash1);
                        p1.Name             = HttpUtilityEx.HtmlDecode(sNick_Name1);
                        p1.ImageUrl         = "http://photo.livede55.com/photo_index/" + sImg1;
                        p1.ImageUpdateCheck = false;
                        p1.Dona             = true;
                        p1.RoomName         = "DS";

                        JSObject jso2 = pair[1] as JSObject;
                        string sHash2      = (string)jso2.GetField("hash",      BindingFlags.Default).GetValue(null);
                        string sNick_Name2 = (string)jso2.GetField("nick_name", BindingFlags.Default).GetValue(null);
                        string sImg2       = (string)jso2.GetField("img",       BindingFlags.Default).GetValue(null);

                        Performer p2 = new Performer(this, sHash2);
                        p2.Name             = HttpUtilityEx.HtmlDecode(sNick_Name2);
                        p2.ImageUrl         = "http://photo.livede55.com/photo_index/" + sImg2;
                        p2.ImageUpdateCheck = false;
                        p2.Dona             = true;
                        p2.RoomName         = "DS";

                        p1.OtherInfo = p2.OtherInfo = p1.Name + "×" + p2.Name;

                        pefs.Add(p1);
                        pefs.Add(p2);
                    }
                } catch (Exception ex) {
                    Log.Add(Site + "-Update失敗(dsroom)", ex.ToString(), LogColor.Error);
                }
            }

            return pefs;
        }

        public FormFlash OpenFlash(Performer performer) {
            //小窓を返す
            return new FormFlashLiveDe55(performer);
        }

        public string GetProfileUrl(Performer performer) {
            //プロフィールURLを返す
            return TopPageUrl + "wait.php?id=" + performer.ID;
        }

        #endregion
    }
}
