﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Diagnostics;
using LiveViewer;
using LiveViewer.Tool;
using LiveViewer.HtmlParser;

namespace Plugin_GOA {
    public class Plugin_GOA : ISitePlugin {
        #region ■オブジェクト

        private TabPage TabPagePlugin;          //設定タブ（今のところなにもなし）
        private Parser  Parser = new Parser();  //HTML解析用パーサ
        private Random  Rnd    = new Random();  //乱数

        //HTML解析用の正規表現
        private Regex RegexGetPefs = new Regex("online(2shot|Party|Wait).", RegexOptions.Compiled);
        private Regex RegexGetID   = new Regex("/u([0-9]*)/", RegexOptions.Compiled);

        #endregion


        #region ■メソッド

        private string GetSubItemValue(HtmlItem hTag, string sTag, string sAttrName, string sAttrValue, bool bHtmlDecode) {
            //タグ(hTag)の配下から、タグ名が(sTag)で属性が(sAttrName)で属性値が(sAttrValue)のタグを探し、その配下の文字列を返す。
            string sRet = null;
            List<HtmlItem> finds = hTag.Find(sTag, sAttrName, sAttrValue, true);
            if (finds.Count > 0) {
                HtmlItem item = finds[0];
                if (item.Items.Count > 0) {
                    if (bHtmlDecode) {
                        sRet = HttpUtilityEx.HtmlDecode(item.Items[0].Text);
                    } else {
                        sRet = item.Items[0].Text;
                    }
                }
            }
            return sRet;
        }

        #endregion


        #region ■ISitePluginインターフェースの実装

        public string Site       { get { return "GOA"; } }

        public string Caption    { get { return "ガールズオンエアー用のプラグイン(2008/07/19版)"; } }

        public string TopPageUrl { get { return "http://www.girlsonair.tv/"; } }

        public void Begin() {
            //プラグイン開始時処理・・設定Tabの生成
            Label label = new Label();
            label.Dock      = DockStyle.Fill;
            label.TextAlign = ContentAlignment.MiddleCenter;
            label.Font      = new Font("MS UI Gothic", 80.0f);
            label.Text      = "ありがとう・・\r\nＧＯＡ(´Д⊂ヽ";

            TabPagePlugin = new TabPage(Site);
            TabPagePlugin.Controls.Add(label);

            Pub.TabControlSetting.TabPages.Add(TabPagePlugin);
        }

        public void End() {
            //プラグイン終了時処理・・設定ページの削除
            Pub.TabControlSetting.TabPages.Remove(TabPagePlugin);
        }

        public List<Performer> Update() {
            //オンラインパフォーマーリスト生成
            List<Performer> pefs = new List<Performer>();

            try {
                //トップページの読み込みと木構造への変換
                Parser.LoadHtml(new Uri(TopPageUrl + "home.jsp"));
                Parser.ParseTree();
                Pub.WebRequestCount++; //GUIの読込回数を増やす
            } catch (Exception ex) {
                //読み込み失敗
                Log.Add(Site + " - Update失敗", ex.ToString());
                return null;
            }

            //パフォ情報のタグを取得
            List<HtmlItem> tagTops = Parser.Find("td", "class", RegexGetPefs); //<td class=onlineWait...>を全て取得

            //各パフォの情報を順に処理
            foreach (HtmlItem tagTop in tagTops) {
                string sID, sImageUrl;
                try {
                    sImageUrl = tagTop.Find("a", "class", "onlinePicture", true)[0].Items[0].GetAttributeValue("src");
                    sID       = RegexGetID.Match(sImageUrl).Groups[1].Value;
                } catch {
                    continue;
                }

                Performer p = new Performer(this, sID);
                pefs.Add(p);

                p.ImageUrl  = sImageUrl;
                p.Name      = GetSubItemValue(tagTop, "div", "class", "onlinePerformerName",  true);
                p.Area      = GetSubItemValue(tagTop, "div", "class", "onlinePerformerProf2", true);
                p.OtherInfo = GetSubItemValue(tagTop, "div", "class", "onlinePerformerProf4", true); 

                if (tagTop.Find("img", "alt", "マイクあり", true).Count != 0) p.Mic     = true;
                if (tagTop.Find("img", "alt", "新人", true).Count != 0)       p.NewFace = true;

                string sTmpDona = tagTop.GetAttributeValue("class").ToLower();
                if (sTmpDona.Contains("wait")) {
                    p.Dona    = false;
                    p.TwoShot = false;
                } else {
                    p.Dona = true;
                    if (sTmpDona.Contains("2shot")) {
                        p.TwoShot = true;
                    } else if (sTmpDona.Contains("party")) {
                        p.TwoShot = false;
                    } else {
                        Log.Add(Site + "Update警告", "未知の状態：" + sTmpDona);
                    }
                }

                try {
                    HtmlItem hi = tagTop.Parent.Parent;
                    if (hi.Name != "div") hi = hi.Parent;
                    if (hi.Name != "div") hi = hi.Parent; //GOAさんDIVタグがおかしい場合がありますよ？？
                    if (hi.Name == "div") {
                        switch (hi.GetAttributeValue("id")) {
                            case "onlineAreaParty": p.RoomName = "ﾊﾟｰﾃｨｰ"; break;
                            case "onlineArea2Shot": p.RoomName = "2ｼｮｯﾄ";  break;
                            default:                p.RoomName = "ERROR";  break;
                        }
                    } else {
                        p.RoomName = "ERROR";
                    }
                } catch {
                    p.RoomName = "ERROR";
                }


                try {
                    string sTmpAge = GetSubItemValue(tagTop, "div", "class", "onlinePerformerProf1", false);
                    if (sTmpAge == "ひみつ") {
                        p.Age = 0;
                    } else {
                        sTmpAge = sTmpAge.Replace("歳",   "");
                        sTmpAge = sTmpAge.Replace("以上", "");
                        p.Age   = int.Parse(sTmpAge);
                    }
                } catch {
                    p.Age = 0;
                }
            }

            return pefs;
        }

        public FormFlash OpenFlash(Performer performer) {
            //フラッシュ窓を返す・・カスタマイズなし
            return new FormFlash(performer);
        }
        
        public string GetFlashUrl(Performer performer) {
            //FlashのURLを返す・・しんびわからパクりましたm(_ _)m
            return TopPageUrl + "flash/goa/freePreview20.swf?performerID=" + performer.ID + "&rank=4&userId=" + Rnd.Next(100000000, 1000000000) + "&photo=http://imageup.girlsonair.tv/WebArchive/u" + performer.ID + "/flash/SelectedImage.jpg&emailNotice=null";
        }

        public Clipping GetFlashClipping(Performer performer) {
            //Flashの切り抜き方法を返す・・GOAは切り抜き不要
            return null;
        }

        public string GetProfileUrl(Performer performer) {
            //プロフィールURLを返す
            return TopPageUrl + "live/index.jsp?id=" + performer.ID;
        }

        #endregion
    }
}
